package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Converting between different length units.</p>
 *
 * <p>Lengths are being measured in different units all over the world. This class considers the following units
 * among with their respective conversions to meter:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Metric and Imperial unit definitions</caption>
 *     <tbody>
 *         <tr>
 *             <th>Name</th>
 *             <th>Enum definition</th>
 *             <th>Length in meter</th>
 *         </tr>
 *         <tr>
 *             <td>Meter (m)</td>
 *             <td>{@link Unit#METER}</td>
 *             <td>1.0</td>
 *         </tr>
 *         <tr>
 *             <td>Inch</td>
 *             <td>{@link Unit#INCH}</td>
 *             <td>0.0254</td>
 *         </tr>
 *         <tr>
 *             <td>Imperial mile</td>
 *             <td>{@link Unit#MILE_IMPERIAL}</td>
 *             <td>1609.344</td>
 *         </tr>
 *         <tr>
 *             <td>Nautical mile</td>
 *             <td>{@link Unit#MILE_NAUTICAL}</td>
 *             <td>1852.0</td>
 *         </tr>
 *
 *     </tbody>
 * </table>
 *
 * <p>The following samples illustrate conversions between different units:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Sample code illustrating unit conversions</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> final LengthUnitCalculator c =
 *      new LengthUnitCalculator();                            // Default unit meter.
 *
 * System.out.println("Turning meter into nautical miles");
 * c.setLength(1852);                                          // Setting length 1852 meter.
 * c.setUnit(Unit.MILE_NAUTICAL);                              // Switching to nautical miles.
 * System.out.println("Nautical miles: " + c.getLength());     // Printing 1852 meter in nautical miles.
 *
 * System.out.println("\nTurning imperial miles into inches");
 * c.setUnit(Unit.MILE_IMPERIAL);
 * c.setLength(2.0);                                           // Two terrestrial miles.
 * c.setUnit(Unit.INCH);                                       // Switching unit to inch.
 * System.out.println("Inches: " + c.getLength());             // Printing 2 terrestrial miles in inches.</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight">Turning meter into nautical miles
 * Nautical miles: 1.0
 *
 * Turning imperial miles into inches
 * Inches: 126720.0</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>You may as well start from a non-meter unit:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Sample code starting from inch default unit.</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> final LengthUnitCalculator c = new LengthUnitCalculator(Unit.INCH); // Default unit inch.
 *
 * System.out.println("Turning inches into meter");
 * c.setLength(1);                                                     // Default unit still inch
 * c.setUnit(Unit.METER);                                              // Switching from inch to meter
 * System.out.println("meter: " + c.getLength());                      // Printing 1 inch in meter.</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight">Turning inches into meter
 * meter: 0.0254</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 */
public class LengthUnitCalculator {

    private Unit unit;      // Currently active unit.
    private double length;  // Length measured in chosen unit.

    /**
     * <p>Setting a default unit.</p>
     * @param unit The default unit to be used.
     */
    public LengthUnitCalculator(final Unit unit) {
        this.unit = unit;
    }

    /**
     * <p>Using default unit {@link Unit#METER}.</p>
     */
    public LengthUnitCalculator() {
        this(Unit.METER);
    }

    /**
     * <p>Switching current unit affecting {@link #setLength(double)} and {@link #getLength()}.</p>
     * @param unit New unit measure.
     */
    public void setUnit(final Unit unit) {
        if (this.unit != unit) { // Optimization
            length *= this.unit.meter / unit.meter;
            this.unit = unit;
        }
    }

    /**
     * <p>Set length measured with respect to currently chosen unit according to {@link #LengthUnitCalculator()},
     * {@link #LengthUnitCalculator(Unit)} or {@link #setUnit(Unit)}.</p>
     * @param length New length in currently chosen unit.
     */
    public void setLength(double length) {
        this.length = length;
    }

    /**
     * <p>Get length with respect to currently active unit.</p>
     * @return Length with respect to currently active unit.
     */
    public double getLength() {
        return length;
    }
}
